from typing import Optional

from pydantic import BaseModel
from whale_client.api.entry import post_application
from whale_client.models.api.entry import PostApplicationRequest, PostApplicationResponse
from whale_client.models.application import Application
import logging

logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)


class Manager(BaseModel):

    def commit(self, application: Application) -> Optional[str]:
        tables_dump: list[dict] = [table.model_dump() for table in application.tables]
        input = PostApplicationRequest(name=application.name, tables=tables_dump)
        response: Optional[PostApplicationResponse] = post_application(input=input)
        if not response:
            log.error("Failed to commit application.")
            return None
        return response.name



import asyncio
from whale_client.models.application import Application
from whale_client.models.table import Column, DataType, PrimaryKey, Table
from whale_client.models.manager import Manager


async def main():

    user_feedback_columns = [
        Column(name="user_id", data_type=DataType.STRING, nullable=False),
        Column(name="application_name", data_type=DataType.STRING, nullable=False),
        Column(name="feedback", data_type=DataType.STRING, nullable=True),
        Column(name="personal_notes", data_type=DataType.STRING, nullable=True),
        Column(name="rating", data_type=DataType.ENUM, enum_values=['1','2','3'], default_value='1')
    ]

    user_feedback_table = Table(
        name="feedback123",
        description="This table stores the user feedback",
        columns=user_feedback_columns,
        primary_key=PrimaryKey.AUTO_INCREMENT,
        enable_created_at_timestamp=True,
        enable_updated_at_timestamp=True,
    )

    user_feedback_tables = [user_feedback_table]

    application_name = "feedback_app11"

    application = Application(application_name, user_feedback_tables)
    manager = Manager()
    response: str = manager.commit(application=application)
    print("SUCCESS")
    
if __name__ == "__main__":
    asyncio.run(main())



import json
from typing import Optional
import requests
from dotenv import load_dotenv

import os

import logging

from whale_client.models.api.entry import PostApplicationRequest, PostApplicationResponse

logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)


load_dotenv()
BASE_URL = os.getenv("BACKEND_ENDPOINT")
SERVICE_ENDPOINT = "application"


def post_application(input: PostApplicationRequest) -> Optional[PostApplicationResponse]:
    try:
        response = requests.post(
            f"{BASE_URL}/{SERVICE_ENDPOINT}", json=input.model_dump()
        )
        response.raise_for_status()
        application_response = PostApplicationResponse.model_validate(json.loads(response.text))
        return application_response
    except TypeError as e:
        log.error(
            f"Failed to parse the response id of application from server: {e}"
        )
        return None
    except requests.RequestException as e:
        log.error(f"Failed to fetch response for application: {e}")
        return None
    except Exception as e:
        log.error(f"Unknown error for posting of application occurred: {e}")
        return None



from enum import StrEnum
from typing import Optional, Any
from pydantic import BaseModel, model_validator
import logging

logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)


class DataType(StrEnum):
    STRING = "string"
    INTEGER = "integer"
    FLOAT = "float"
    BOOLEAN = "boolean"
    DATE = "date"
    # DATETIME = "datetime"
    UUID = "uuid"
    ENUM = "enum"


class PrimaryKey(StrEnum):
    AUTO_INCREMENT = "auto_increment"
    UUID = "uuid"


class ForeignKey(BaseModel):
    table: str
    column: str


class Column(BaseModel):
    name: str
    data_type: DataType
    enum_values: Optional[list[str]] = None
    nullable: bool = False
    default_value: Optional[Any] = None
    unique: Optional[bool] = False
    foreign_key: Optional[ForeignKey] = None

    @model_validator(mode="before")
    @classmethod
    def set_default_value(cls, data: Any) -> Any:

        if not isinstance(data, dict):
            raise ValueError("Column data must be a dictionary.")

        cls._validate_enum_values(data)
        cls._set_default_value(data)

        return data

    @staticmethod
    def _validate_enum_values(data: dict) -> None:
        if data.get("data_type") == DataType.ENUM:
            if data.get("enum_values") is None:
                raise ValueError(
                    "enum_values must be set for columns with data type 'enum'."
                )
            if not data.get("enum_values"):
                raise ValueError("enum_values cannot be empty.")
            if len(data.get("enum_values")) != len(set(data.get("enum_values"))):
                raise ValueError("enum_values must be unique.")

            non_string_values = [
                v for v in data.get("enum_values") if not isinstance(v, str)
            ]
            if non_string_values:
                raise ValueError(
                    f"All enum_values must be of type string. Non-string values found: {non_string_values}"
                )

            if data.get("default_value") not in data.get("enum_values"):
                raise ValueError("default_value must be one of the enum_values.")
        else:
            if data.get("enum_values"):
                raise ValueError(
                    "enum_values can only be set for columns with data type 'enum'."
                )

    @staticmethod
    def _set_default_value(data: dict) -> None:
        # If the default value is set, use it
        if "default_value" in data:
            return

        data_type = data["data_type"]
        if data_type == DataType.STRING:
            data["default_value"] = ""
        elif data_type == DataType.INTEGER:
            data["default_value"] = 0
        elif data_type == DataType.FLOAT:
            data["default_value"] = 0.0
        elif data_type == DataType.BOOLEAN:
            data["default_value"] = False
        elif data_type == DataType.DATE:
            data["default_value"] = "1970-01-01"
        # elif data_type == DataType.DATETIME:
        #     data["default_value"] = "1970-01-01T00:00:00Z"  # ISO format for datetime

        return data

    def __init__(self, **data):
        super().__init__(**data)
        self._validate_name()

    def _validate_name(self):
        if not self.name:
            raise ValueError("Column name cannot be empty.")

        if not self.name.islower():
            raise ValueError("All characters in column name must be in lower case.")

        if " " in self.name:
            raise ValueError("Column name cannot contain spaces.")


class Table(BaseModel):
    name: str
    description: Optional[str] = None
    columns: list[Column]
    primary_key: PrimaryKey = PrimaryKey.AUTO_INCREMENT
    enable_created_at_timestamp: Optional[bool] = False
    enable_updated_at_timestamp: Optional[bool] = False

    def __init__(self, **data):
        super().__init__(**data)
        self._validate_name()

    def _validate_name(self):
        if not self.name:
            raise ValueError("Table name cannot be empty.")

        if not self.name.islower():
            raise ValueError("All characters in table name must be in lower case.")

        if " " in self.name:
            raise ValueError("Table name cannot contain spaces.")



from dataclasses import dataclass
from whale_client.models.table import Table

VERSION = "v1"

@dataclass
class Application:
    
    _name: str
    _tables: list[Table]
    
    def __init__(self, name: str, tables: list[Table]):
        self._name = name
        self._tables = tables
        self._validate_name()
        self._validate_tables()
        
        
    def _validate_name(self):
        if not self._name: 
            raise ValueError("Application name cannot be empty.")
        
        if not self._name.islower():
            raise ValueError("All characters in application name must be in lower case.")
        
        if " " in self._name:
            raise ValueError("Application name cannot contain spaces.")
        
    def _validate_tables(self):
        if not self._tables:
            raise ValueError("Application must contain at least one table.")
        
        table_names = [table.name for table in self._tables]
        if len(table_names) != len(set(table_names)):
            raise ValueError("Table names passed into Application are not unique.")
        
        
    @property  
    def name(self) -> str:
        return self._name
    
    @property
    def tables(self) -> list[Table]:
        return self._tables
        
    def __str__(self):
        return f"Application ({self.name}) contains {len(self.tables)} tables: {', '.join([table.name for table in self.tables])}"
    